using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Intralogic.Compilers.VisualBNF;

namespace WindowsApplication1
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            textBox2.Text = "";
            textBox3.Text = "";

            // create error list instance that parser will write to.
            List<CompilerError> errors = new List<CompilerError>();

            // create compiler input instance that parser will use for parse directions
            CompilerInput compilerInput = new CompilerInput();

            // create compiler internals instance which can be found in email.dll (internals, tables etc)
            CompilerInternals compilerInternals = new CompilerInternals();

            // copy internals members to compiler input for parser
            compilerInput.DFAMultiStopTable = compilerInternals.DFAMultiStopTable;
            compilerInput.DFAStopTable = compilerInternals.DFAStopTable;
            compilerInput.DFATable = compilerInternals.DFATable;
            compilerInput.DFAUnicodeTable = compilerInternals.DFAUnicodeTable;
            compilerInput.LRTable = compilerInternals.LRTable;
            compilerInput.LRTableDotCount = compilerInternals.LRTableDotCount;
            compilerInput.LRTableDotPosition = compilerInternals.LRTableDotPosition;
            compilerInput.ProductionSymbols = compilerInternals.ProductionSymbols;
            compilerInput.ReductionProds = compilerInternals.ReductionProds;
            compilerInput.ReductionSize = compilerInternals.ReductionSize;
            compilerInput.TerminalEofIndex = compilerInternals.TerminalEofIndex;
            compilerInput.TerminalSymbols = compilerInternals.TerminalSymbols;
            compilerInput.TerminalWSIndex = compilerInternals.TerminalWSIndex;

            // create parser
            LRParser parser = new LRParser(compilerInput, errors);

            // set source text for parser
            parser.SetSource(textBox1.Text);

            // start parse source
            LRParser.Result result = parser.Parse();

            // test parser for result
            if (result == LRParser.Result.NotAccepted)
                MessageBox.Show("Parser does not accept source text");
            else if(result == LRParser.Result.AcceptedWithErrors)
                MessageBox.Show("Parser accepted source text but there was some errors");
            else if(result == LRParser.Result.Accepted)
                MessageBox.Show("Parser accepted source text");

            // get syntax tree from parser instance
            ParseTree parserTree = parser.Tree;

            // cut chain of non-terminals in tree
            parserTree.CutAST(parserTree);

            // generate proper indexes and parents of tree
            parserTree.RebuildTree(parserTree, 0);
            
            // convert offsets to columns and rows
            parser.CalculateColumnsAndRowsInErrorList(errors, textBox1.Text);

            // write errors if any to output
            for (int i = 0; i < errors.Count; i++)
            {
                textBox3.Text += errors[i].ToString() + "\r\n";
            }

            // search tree for email tokens
            TreeSearch search = new TreeSearch();
            search.MarkStartPoint(parser.Tree, "email", true);
            ParseTree treeWalk = null;
            while (null != (treeWalk = search.Find()))
            {
                textBox2.Text += treeWalk.ValueName + " ";
            }
        }
    }
}